package org.telosys.tools.commons.jdbctypes;

import java.util.Iterator;
import java.util.LinkedList;


/**
 * List of JdbcType instances <br>
 * List of associations "JDBC code" - "code text" + "recommended Java type" 
 * @author L. Guerin
 *
 */
public class JdbcTypes {
	
	private LinkedList<JdbcType> _list = new LinkedList<JdbcType>() ;
	
	public JdbcTypes() {
		super();
	}

	/**
	 * Adds a new JDBC type in the list
	 * @param code the JDBC code 
	 * @param text the JDBC text ( Java constant name from 'java.sql.Types' : "NUMERIC", "TINYINT", "VARCHAR", ... )
	 * @param type the recommended Java type associated with the JDBC code 
	 */
	public void add(int code, String text, String type)
	{
		_list.add( new JdbcType(code, text, type) );
	}
	
	/**
	 * Adds a new JDBC type in the list with different types for "NULL" and "NOT NULL"
	 * @param code
	 * @param text
	 * @param typeForNull the recommended Java type associated with the JDBC code if "NULL"
	 * @param typeForNotNull the recommended Java type associated with the JDBC code if "NOT NULL"
	 */
	public void add(int code, String text, String typeForNull, String typeForNotNull)
	{
		_list.add( new JdbcType(code, text, typeForNull, typeForNotNull) );
	}
	
	/**
	 * Returns the Java constant name ( from 'java.sql.Types' ) for the given JDBC code 
	 * @param code
	 * @return the text or null if unknown code
	 */
	public String getTextForCode( int code )
	{
		Iterator iter = _list.iterator();
		while ( iter.hasNext() )
		{
			JdbcType t = (JdbcType) iter.next() ;
			if ( t.getCode() == code )
			{
				return t.getText() ;
			}
		}
		return null ;
	}
	
	/**
	 * Returns the recommended Java type associated with the given JDBC code 
	 * @param iJdbcTypeCode
	 * @return the Java type ( primitive type or fully qualified class name )<br>
	 * e.g. : "int", "java.lang.String", "byte[]", "java.sql.Time"
	 */
	public String getJavaTypeForCode( int iJdbcTypeCode, boolean bNotNull )
	{
		Iterator iter = _list.iterator();
		while ( iter.hasNext() )
		{
			JdbcType t = (JdbcType) iter.next() ;
			if ( t.getCode() == iJdbcTypeCode )
			{
				if ( bNotNull ) {
					return t.getJavaTypeForNotNull() ;
				}
				else {
					return t.getJavaTypeForNull() ;
				}
			}
		}
		return null ;
	}
	
	/**
	 * Returns the list size
	 * @return
	 */
	public int size()
	{
		return _list.size();
	}
}
